function z0 = getsf(p, x, y, varargin),

% GETSF return surface details for the primitive. 
% (part of the RAYTRACE toolbox.)
%
% GETSF(p, x, y, ...). Here p is the primitive, (x,y) is the position and 
% additional parameters are used to specify height or derivative. 
%
% EXAMPLES 
%
%   r0=7.5;a=0.0125;r1=1.5;r2=3.0; x = 1.5; y = 1.5;
%   p = prim('rkklein','klein-RK','on',[0 0 0],[0 10 0],4,1e-9,r0,a,r1,r2); 
%   z    = getsf(p, x, y);          % height
%   dzdx = getsf(p, x, y, 'dzdx');  % dz/dx
%


% get posn. in shfted/rotated space.
x = x - p.position(1);
y = y - p.position(2);
nOrient = (p.orientation == [ 0 0 0 ]);

if (all(nOrient)), % no orientation 
    z0 = feval(p.type, x, y, p.args.value{:},varargin{:});
    if (nargin == 3),
        z0 = z0 + p.position(3);
        return
    end
    if (strcmp(varargin{1}, 'z')), 
        z0 = z0 + p.position(3);
        return
    end
    return
end

% p.orientation

% orientation requested.
etol   = 10;  % maximum tolerance
nR     = getRotMtx('zyx', degtorad(p.orientation)); nR = inv(nR);
nPos   = [ x y x*0-etol ];
nSlope = zeros(size(x,1),3); nSlope(:,3) = 1;

% rotate into canonical frame
nPos      = (nR*(nPos'))';
nSlope    = (nR*(nSlope'))';

if (isnan(p.etol)),
    [z, j]   = nsect(p.type, nPos, nSlope, p.aperture, p.args.value{:});
else
    [z, j]   = nsect(p.type, nPos, nSlope, p.aperture, p.etol, p.args.value{:});    
end

n        = size(nPos,1);

if (~isempty(varargin)),
    switch(varargin{1}),
        case { 'z' },
            z = inv(nR)*(z'); z = z';
            z = z(:,3) + p.position(3);
            z0 = nan*ones(n,1);
            z0(j) = z;
            return
        case { 'dzdx' }
            z = feval(p.type, z(:,1), z(:,2), p.args.value{:},'normal');
            z = inv(nR)*(z'); z = z';
            z = -z(:,1)./z(:,3);
            z0 = nan*ones(n,1);
            z0(j) = z;
            return
        case { 'dzdy' }
            z = feval(p.type, z(:,1), z(:,2), p.args.value{:},'normal');
            z = inv(nR)*(z'); z = z';
            z = -z(:,2)./z(:,3);
            z0 = nan*ones(n,1);
            z0(j) = z;
            return
        case  { 'normal' }
            z = feval(p.type, z(:,1), z(:,2), p.args.value{:},'normal');
            z = inv(nR)*(z'); z = z';
            z(:,1) = -z(:,1)./z(:,3);
            z(:,2) = -z(:,2)./z(:,3);
            z(:,3) = -1;
            z0 = nan*ones(n,3);
            z0(j,:) = z;
            return
        otherwise,
            error('(GETSF) Unknown invocation parameter.');
    end 
else
    % default to height
    z = inv(nR)*(z'); z = z';
    z = z(:,3) + p.position(3);
    z0 = nan*ones(n,1);
    z0(j) = z;
    return
end
    
return