function s = ray(varargin),

% RAY provide a rays class.
%
% RAY(<source position>, <slope>). Generate rays for 
% source and slope.  If the number of source rows does not 
% equal the slope rows then all combinations of source and slope 
% will be generated.
%
% RAY('AIM-XYZ', <source position>, <target position). Generate rays that  
% start at source and are aimed at the target.  Make all combinations 
% of source and target positions. 
%
% RAY('AIM-PL', <source position>, nPhi, nTheta). Generate rays that  
% start at source and are aimed at the target.  Make all combinations 
% of source and target positions. 
%
% RAY('FAN-YZ', <source position>, nRays, nAngle). Create 
% a fan of nRays equally spaced rays that subtend angle 2*nAngle
% confined to the YZ plane. Fan emanates from source position. 
% 
% RAY('FAN-XZ', <source position>, nRays, nAngle). Create 
% a fan of nRays equally spaced rays that subtend angle 2*nAngle
% confined to the XZ plane.  Fan emanates from source position.  
%
% RAY('GRID-FAN', <source position>, nRays, nAngle). A fan grid   
% of nRays over the equally spaced rays that subtend angle 2*nAngle
% in the XZ plane.  Grid fan emanates from source position. 
%
% RAY('GRID-SQ', <source position>, nTgtCenter, nBoxSide, nRaysPerSide).
% A fan grid of nRays over the equally spaced rays that subtend angle 2*nAngle
% in the XZ plane.  Grid fan emanates from source position. 
%
%
% see also PRINT, ISEMPTY 
%
% Examples 
%
% load a ray with position and slope directly
%
%    r = ray([0 0 0], [ 0 0 2 ; 0 0 -1 ]);
%    print(r); 
%
%        x1	       x2	        x3	        x4	        x5	        x6	       OPL
% 1.	0.000000	0.000000	0.000000	0.000000	0.000000	1.000000	0.000000
% 2.	0.000000	0.000000	0.000000	0.000000	0.000000	-1.000000	0.000000
%
% load ray specifying source and target 
%
%


s = init_class;

if (nargin == 0),
   return
end 

if (nargin == 2)&(isnumeric(varargin{1}))&(isnumeric(varargin{2})),

   nPos   = varargin{1};
   nSlope = varargin{2};
       
   if (size(nPos,1) ~= size(nSlope, 1)),
      [nPos, nSlope]   = mesh3vec(nPos, nSlope);
   end

   % simple position and ray slope defined
   s.position   = nPos;
   s.slope      = vzunit(nSlope);  % unit 
   s.OPL        = zeros(size(nPos,1), 1);
   s.index      = 1;
   return
end 

if (~ischar(varargin{1})),
    error('(RAY) Unknown invocation used.');
end
   
% keyword used 
switch(varargin{1}),
    % RAYS('AIM-XYZ', <source position>, <target position>)
    case 'AIM-XYZ',
       nPos = varargin{2};   % source 
       nTgt = varargin{3};   % target    
       if (size(nPos, 1) ~= size(nTgt, 1)),
          [nPos, nTgt]   = mesh3vec(nPos, nTgt);
       end
       
       if (nargin == 3),
          [s.position, s.slope] = createray(nPos, nTgt);
       else
          [s.position, s.slope] = createray(nPos, nTgt, varargin{:});
       end
          
    % RAYS('AIM-PL', <source position>, nPhi, nTheta)
    case 'AIM-PL',

       theta  = varargin{3}; phi  = varargin{4}; 
       nSlope = [ cos(theta).*sin(phi) sin(theta) cos(theta).*cos(phi) ]; 
       nPos   = varargin{2};   
       
       if (size(nSlope,1) ~= size(nPos,1)),
          [nPos, nSlope]   = mesh3vec(nPos, nSlope);
       end
       
       s.position = nPos;
       s.slope    = nSlope;
               
    % RAYS('FAN-YZ', <source position>, nRays, nAngle) 
    case 'FAN-YZ',
       nPos     = varargin{2};
       nRay     = varargin{3};
       nAng     = varargin{4};
                
       theta    = linspace(-nAng, nAng, nRay)';
       s.position   = nPos(ones([ length(theta) 1]),:);
       s.slope  = [ 0*theta sin(theta) cos(theta) ]; 

    % RAYS('FAN-XZ', <source position>, nRays, nAngle)    
    case 'FAN-XZ',
       nPos     = varargin{2};
       nRay     = varargin{3};
       nAng     = varargin{4};
       
       theta    = linspace(-nAng, nAng, nRay)';
       s.position   = nPos(ones([ length(theta) 1]),:);
       s.slope  = [ sin(theta) 0*theta cos(theta) ]; 

       
    % RAYS('GRID-SQ', <source position>, nTgtCenter, nBoxSide, nRaysPerSide)
    case 'GRID-SQ',
        
       nPos         = varargin{2};
       nTgtCenter   = varargin{3};
       nBoxSide     = varargin{4};
       nRaysPerSide = varargin{5};
              
       nX = linspace(-nBoxSide, nBoxSide, nRaysPerSide); 
       [ nX, nY ] = meshgrid(nX);
       nX = nTgtCenter(1) + nX(:); nY = nTgtCenter(2) + nY(:);
       
       if (size(nPos,1) > 1),
          error('(RAY/GRID-SQ) Only one source position allowed here.'); 
      else
          n0   = size(nX,1);
          nPos = nPos(ones([n0 1]), :);
          [s.position, s.slope] = createray(nPos, [nX nY 0*nX+nTgtCenter(3)]);
       end
       
       
    % RAYS('GRID-FAN', <source position>, nPos, nRay, nAng)
    case 'GRID-FAN',
        
       nPos     = varargin{2};
       nRay     = varargin{3};
       nAng     = varargin{4};
       
       phi          = linspace(-nAng, nAng, nRay);
       [phi, theta] = meshgrid(phi, phi);
       phi = phi(:); theta = theta(:); 
       
       s.position   = nPos(ones([ size(phi,1) 1 ]),:); 
       s.slope  = [ cos(theta).*sin(phi) sin(theta) cos(theta).*cos(phi) ]; 
       
    case 'AXIAL-BEAM',
        
       nS       = varargin{1};
       nAp      = varargin{2};
       nGrid    = varargin{3};
       nR       = -nAp/nS;      % entrance beam
              
       t_x        = linspace(-nR, nR, nGrid);
       [t_x, t_y] = meshgrid(phi, phi); 
       t_x = t_x(:); t_y = t_y(:); 
       
       s.position   = [ 0 0 nS ]; 
       s.position   = s.position(ones([ size(phi,1) 1 ]),:); 
       s.slope      = [ cos(t_y).*sin(t_x) sin(t_y) cos(t_x).*cos(t_y) ]; 
       
       
       
       
    otherwise, 
        error('(RAY) Unknown option requested.');
end 

% assumed information
nRays = size(s.position, 1);
s.OPL   = zeros(nRays, 1);
s.index = 1;
       

return

function s = init_class,

    s.position      = [];
    s.slope         = [];
    s.OPL           = [];
    s.index         = [];
    s           = class(s, 'ray');
return 