function n = cauchy(x, A, B, C, D),

% CAUCHY. Implement the Cauchy chromatic dispersion formula 
%
% CAUCHY(x, A, B, C, D). Here x is the wavelength in nanometers, 
% (A, B, C, D) are the coefficients as described below. 
% 
% The Cauchy formula is given by 
%
%  n(x) = A + B/x^2 + C/x^4 + D/x^6 
%
% The equivalent Herzberger (see HERZBERGER) coefficients are 
%
%   Medium        A           B           C           D 
%   --------------------------------------------------------
%   Cornea    1.362017  7.001974e3  -1.020784e9  9.755871e13
%   Aqueous   1.322776  7.407115e3  -1.100281e9  1.035667e14
%   Lens      1.404662  6.908333e3  -8.200728e8  9.356781e13
%   Vitreous  1.322781  6.392004e3  -8.508829e8  9.127679e13
%
% Information taken from : 
% Smith, G. and Atchison, DA. "Chromatic dispersions of the ocular 
% media of human eyes." JOSA A (2005), 22(1), 29 -- 37. 
%
% 
% Example, [ compare errors at 900 nm ]
% 
%   % cf. two methods at 900 nm (herzberger (micron)/ cauchy (nm))
%   % Smith and Atchison suppose a 0.003 diff. 
%   nh = herzberger(900/1000, 1.3975, 1.3807, 1.37405, 1.3668);
%   nc = cauchy(900, 1.362017, 7.001974e3, -1.020784e9, 9.755871e13);
%   fprintf('NH = %6.6f  NC = %6.6f DF = %6.6f\n', nh, nc, nc-nh);
%

n = A + B./x.^2 +  C./x.^4 +  D./x.^6;

return