function r = getEntrBeam(p, s0, nID, nAp, etol),

% GETENTRBEAM Estimate the direction cosines of the beam
% entering the optical system and passing through the 
% aperture stop. The direction cosines are the components 
% of the unit vector.
%
% GETENTRBEAM(p, s0, nId, etol). Here p is the lens prescription, 
% s0 is the source point, etol is the tolerance and nID is  the 
% aperture stop.
%
% GETENTRBEAM(p, s0, nId, nAp, etol). Here p is the lens prescription, 
% s0 is the source point, etol is the tolerance and nID is  the 
% aperture stop. nAp is the aperture (different possibly from the defined).
%
%
% NOTE :
%      1/ Source is assumed to 
%      2/ Pupil surface is assumed to be centered.
%      3/ It is possible that sending the rays through the lens 
%         prescription will result in .
%       
%
% EXAMPLE
%
%  x = test_eye(1);  % le grand 
%  y = getEntrBeam(x, [0 0 -10], 6, 1e-6);
%   
%  >> r
%
%  r =
%
%         0    0.0365    0.9993     <---- horizontal dirn. cosine
%         0   -0.0365    0.9993
%    0.0365         0    0.9993     <---- vertical dirn. cosine     
%   -0.0365         0    0.9993
%

p   = seek(p, nID+1);
p   = split(p, 'front');
p   = seek(p, nID);

nPr = get(p);

if (nargin == 4),
    nAp = get(nPr, 'aperture');
end

nZ  = get(nPr, 'position'); 
if ((nZ(1)~=0)|(nZ(2)~=0)),
    error('(GETENTRBEAM) Not Implemented. NON-CENTERED PUPIL.');
    return
end  
nZ  = nZ(3);

p0  = setApInf(p);    % dissolve apertures
nAp = nAp-etol;     % we want a better approx. to apertures
x0  = [ 0 nAp nZ ; 0 -nAp nZ ; nAp 0 nZ ; -nAp 0 nZ ]; 
s0  = s0(ones(4,1), :);
r0   = rayloc(p0, etol, s0, x0);
r    = vunit(get(r0, 'slope'));

% ... this will fail if GETENTRBEAM wasnt successful.
% print(r0);
trace(r0, p);

return