function x = eye_navarro(varargin),

% EYE_NAVARRO. The Navarro eye as implemented in the paper 
% R. Navarro, J. Santamaria, and J. Bescos, "Accommodation-
% dependent model of the human eye with aspherics," 
% J. Opt. Soc. Am. A 2(8), 1273-1281 (1985)
% 
% Adjustments as presented in : Escudero-Sanz and R. Navarro, 
% "Off-axis aberrations of a wide-angle schematic eye model ,"  
% J. Opt. Soc. Am. A 16(8), 1881-1891 (1999)
%
%
% EYE_NAVARRO('Property1','Property1Value', ...). Set properties for 
% the navarro eye.
%
% Properties 
%
%   FocalLength   = 24.0 mm
%   PupilDiameter = 2.0 mm
%
%
%   #   n       CV          CC      d       p       description
% ---------------------------------------------------------------
%       1.0000                                      air
%   1           1/7.72      -0.26                   
%       1.376                       0.55    0.55    cornea
%   2           1/6.5       0                                  
%       1.3374                      3.05    3.6     aqueous
%   3           inf         0                                  
%       1.3374                      0       3.6     stop
%   4           1/10.000    -3.1316                   
%       1.4200                      4.0     7.6     lens
%   5           -1/6.000    -1                   
%       1.336                                       vitreous
%
% refractive indices have been computed for wavelength 589.3 nm. 
% The corneal ref. index of 1.376 has been used. An IMG plane will 
% be added at the paraxial image plane.
%
% NOTE : 
%
% 1/ defaults to 589.3 nm, but ref. indices are available in the code
%
%   Medium    458nm   543nm  589.3nm  632.8nm
%   ----------------------------------------- 
%   Cornea    1.3828  1.3777  1.376   1.3747
%   Aqueous   1.3445  1.3391  1.3374  1.336
%   Lens      1.4292  1.4222  1.42    1.4183
%   Vitreous  1.3428  1.3377  1.336   1.3347
%
% REFERENCES : 
% 
% 1. Escudero-Sanz and R. Navarro, "Off-axis aberrations of a wide-angle schematic 
% eye model ,"  J. Opt. Soc. Am. A 16(8), 1881-1891 (1999)
% 
% 2. R. Navarro, J. Santamaria, and J. Bescos, "Accommodation-dependent model of the 
% human eye with aspherics," J. Opt. Soc. Am. A 2(8), 1273-1281 (1985)
% 

if (nargin > 0)
    nAp = getProperty('PupilDiameter', varargin, inf)/2;
    f2  = getProperty('FocalLength',varargin, 24);
else
    nAp = inf; % mm
    f2 = 24.0; 
end

% refractive indices for wavelength
n = [ 458   1.3828 1.3445 1.4292 1.3428 ; ...
      543   1.3777 1.3391 1.4222 1.3377 ; ... 
      589.3 1.376  1.3374 1.42   1.336  ; ...
      632.8 1.3747 1.336  1.4183 1.3347 ];

% default to 589.3 nm
n = n(3,:); 
  
x = optic;
x = addon(x, 'index','air',1); 
x = addon(x, 'conic', 'anterior cornea', 'refract', [0 0 0], [0 0 0], 7.72, 1/7.72, -0.26); 
x = addon(x, 'index','cornea', n(2)); 
x = addon(x, 'sphere', 'posterior cornea', 'refract', [0 0 0.55], [0 0 0], 6.5, 6.5); 
x = addon(x, 'index','aqueous', n(3)); 
% x = addon(x, 'pupil', 'pupil', 'refract', [0 0 3.6], [0 0 3.6], nAp);   
% x = addon(x, 'index','lens', n(3));
x = addon(x, 'conic', 'anterior lens', 'refract', [0 0 3.6], [0 0 0], nAp, 1/10.2, -3.1316);   
x = addon(x, 'index','lens', n(4));
x = addon(x, 'conic', 'posterior lens', 'refract', [0 0 7.6], [0 0 0], 6, -1/6, -1);     
x = addon(x, 'index','vitreous',n(5));
x = addon(x, 'img','image','refract',[0 0 f2], [0 0 0],inf);
x = gohead(x); 

return
