function [r, j] = raynsect(r, p, varargin),

% RAYNSECT Intersect a surface with a ray.
%
% [r, j] = RAYNSECT(r, p). Intersect ray (r) with the optical surface (p). 
% This assumes a refractive index of n = 1.
%
% [r, j] = RAYNSECT(r, p, 'fwd' OR 'bck' ). Intersect ray (r) with the 
% optical surface (p). Selecting 'fwd' or 'bck' enables reverse or 
% forward tracing to the surface. This assumes a refractive index of n = 1.
%
% [r, j] = RAYNSECT(r, p, n1, 'fwd' OR 'bck'). The refractive index n1 is 
% added into the mix. Selecting 'fwd' or 'bck' enables reverse tracing 
% to the surface.

if (nargin == 2),
   n1 = 1;
   strDirn = 'fwd';
elseif (nargin == 3),
    if (ischar(varargin{1})),
        n1 = 1;
        strDirn = varargin{1};
    elseif (isnumeric(varargin{1})),
        n1 = varargin{1};
        strDirn = 'fwd';
    else
        error('(RAYNSECT) Unknown invocation.');
    end 
elseif (nargin == 4),
   n1 = varargin{1};
   strDirn = varargin{2};
end


nAng      = get(p, 'orientation');  % canonical frame 
nOrg      = get(p, 'position');
r         = shift(r, -nOrg);    
R         = getRotMtx('zyx', degtorad(nAng));  R =inv(R);
r         = rotate(r, R);    

% trace to canonical surface 
nAp    = get(p,'aperture');
nArgs  = get(p,'args');
strFn  = get(p,'type');
nPos   = get(r,'position');
nSlope = get(r,'slope');
nOPL   = get(r,'OPL');

if (strcmp(strDirn,'bck')),
    nSlope = -nSlope;
end

% intersect the canonical surface
nPosOld = nPos;
etol = get(p,'etol');
if (isnan(etol)),
    [nPos, j]   = nsect(strFn, nPos, nSlope, nAp, nArgs{:});
else
    [nPos, j]   = nsect(strFn, nPos, nSlope, nAp, etol, nArgs{:});    
end

nPosOld = nPosOld(j,:); nSlope = nSlope(j,:); nOPL = nOPL(j,:);


if (strcmp(strDirn,'bck')),  % reverse 
    nOPL    = -n1*vnorm(nPos - nPosOld) + nOPL;
    nSlope  = -nSlope;
elseif (strcmp(strDirn,'fwd')), % standard
    nOPL    = n1*vnorm(nPos - nPosOld) + nOPL;
end

% clean-up  
r  = set(r,'slope', nSlope);            
r  = set(r,'position', nPos);           
r  = rotate(r, inv(R));
r  = shift(r, nOrg);
r  = set(r, 'OPL', nOPL);
return