function [r, p, x] = step(r, p)

% STEP perform one ray-tracing step.
%
% [r, p, x] = STEP(p, r).  Step through one ray-trace returning 
% a ray object (r) and optical object (p) and an index
% showing remaining rays. 
%
% returns a structure with information regarding the
% current STEP. 
%
%  x.optic  = current PRIM 
%  x.n1     = input refractive index 
%  x.n2     = output refractive index 
%  x.index  = indices of surviving rays 

% --------------------------------------------------
% notes:
% 24/11/04 added an OPL section by modifying "rays"
% Bug idetified 20/10/04 - in toric (doesnt like rh=rv)
% Tues 9th Oct 2001 -  optic decenter not supported.
% Dec. 5th 2001     -  optic decenter implemented.
% ---------------------------------------------------

if (~isa(r, 'ray')|~isa(p, 'optic')),
    error('(STEP) Correct invocation is STEP(ray, optic)');
end


% get the first INDEX
n1  = get(p);
n1  = get(n1, 'args', 'index'); 
r   = set(r, 'index', n1);      

% prepare surface 
p         = next(p);                    
myOpt     = get(p);
nAng      = get(myOpt,'orientation');  % canonical frame 
nOrg      = get(myOpt,'position');
r         = shift(r, -nOrg);    
R         = getRotMtx('zyx', degtorad(nAng));  R =inv(R);
r         = rotate(r, R);    

% trace to canonical surface 
nAp    = get(myOpt,'aperture');
nArgs  = get(myOpt,'args');
strFn  = get(myOpt,'type');
nPos   = get(r,'position');
nSlope = get(r,'slope');
nOPL   = get(r,'OPL');

% intersect the canonical surface
nPosOld = nPos;

% [ nPos, j ] = nsect(strFn, nPos, nSlope, nAp, nArgs{:});
% fprintf('INPUT NPOS/NSLOPE\n');
% [ nPos nSlope ]
% print(myOpt);

etol = get(myOpt,'etol');
if (isnan(etol)),
    [nPos, j]   = nsect(strFn, nPos, nSlope, nAp, nArgs{:});
else
    [nPos, j]   = nsect(strFn, nPos, nSlope, nAp, etol, nArgs{:});    
end


if  isempty(j),
    fprintf('-------------------------------------------------------\n');
    fprintf('(STEP) Halted prematurely. No rays reached surface.\n');
    print(myOpt);
    fprintf('waiting ...\n');
    keyboard;
end  

nPosOld = nPosOld(j,:); nSlope = nSlope(j,:); nOPL = nOPL(j,:);
nOPL    = n1*vnorm(nPos - nPosOld) + nOPL;

% fprintf('OUTPUT NPOS/NSLOPE\n');
% [ nPos nSlope ]
% keyboard;


% fprintf('stopped at nsect');
% keyboard;

% IMG causes instant return
if (strcmp(strFn, 'img')),
    % clean-up.
    r  = set(r,'slope', nSlope);            
    r  = set(r,'position', nPos);            
    r  = rotate(r, inv(R));
    r  = shift(r, nOrg);
    r  = set(r, 'OPL', nOPL);

    x.prim     = myOpt;
    x.indexes  = j;
    x.n1       = n1;
    x.n2       = n1;
    return     
end 

% get the next index
p      = next(p);
n2     = get(p);                 
n2     = get(n2, 'args', 'index'); % get index or error

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Refraction/Reflection 
% 
% NB : no rays will be discarded.

% get normals [ pupil and image are trivial ]
n  = feval(strFn, nPos(:,1), nPos(:,2), nArgs{:}, 'normal');
nSlope  = bend(get(myOpt,'refraction'), nSlope, n, n1, n2);   % do refract/reflect 

% clean-up  
r  = set(r,'slope', nSlope);            
r  = set(r,'position', nPos);           
r  = rotate(r, inv(R));
r  = shift(r, nOrg);
r  = set(r, 'OPL', nOPL);

% fprintf('END OF STEP.\n');
% print(r);

x.prim     = myOpt;
x.indexes  = j;
x.n1       = n1;
x.n2       = n2;
return